from django.contrib.contenttypes.models import ContentType

from rest_framework import serializers

from plugs_media.models import Media


class MediaSerializer(serializers.ModelSerializer):
    """
    Media Serializer
    """

    file = serializers.FileField(
        max_length=100,
        allow_empty_file=False,
        use_url=False
    )

    class Meta:
        model = Media
        fields = ('id', 'name', 'file', 'content_type', 'object_id', 'created', 'updated')

class MediaContentTypeSerializer(serializers.ModelSerializer):
    """
    Media Serializer
    """

    class Meta:
        model = ContentType
        fields = ('id', 'app_label', 'model')



"""
Plugs Media Validators
"""

from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext as _
from django.core.exceptions import ValidationError

from plugs_media.models import Media

# pylint: disable=R0903
class MediaValidator(object):
    """
    Validates if a string passed to a field
    accepting media resources is in fact a media
    resource
    """

    message = _('This field must be a valid media resource.')
    code = 'invalid_media'

    def __call__(self, value):
        try:
            Media.objects.get(file=value)
        except ObjectDoesNotExist:
            raise ValidationError(self.message, code=self.code)



"""
Plugs media models
"""

from django.db import models
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from plugs_core import mixins

class Media(mixins.Timestampable, models.Model):
    """
    Media Model
    """
    name = models.CharField(max_length=255, null=True, blank=True)
    file = models.FileField(max_length=255, upload_to='%Y/%m/%d/')
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True)
    object_id = models.PositiveIntegerField(null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    # pylint: disable=R0903
    class Meta:
        """
        Metaclass definition
        """
        verbose_name = 'media'
        verbose_name_plural = 'media'



"""
Plugs Media Views
"""
from django.contrib.contenttypes.models import ContentType

from rest_framework import permissions
from rest_framework.viewsets import ReadOnlyModelViewSet

from plugs_core import viewsets

from plugs_media import models
from plugs_media import serializers

from plugs_filter.decorators import auto_filters

@auto_filters
class MediaViewSet(viewsets.CreateReadViewSet):
    """
    Media Endpoint, does not support update
    or delete operations
    """
    queryset = models.Media.objects.all()
    serializer_class = serializers.MediaSerializer
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    auto_filters_fields = ('content_type', 'object_id')

class MediaContentTypeViewSet(ReadOnlyModelViewSet):
    """
    Exposing Content Types
    """
    queryset = ContentType.objects.all()
    serializer_class = serializers.MediaContentTypeSerializer
    permission_classes = [permissions.AllowAny]
